@@ -1,6 +1,6 @@
 import time
 from typing import List, Optional, Tuple
-from pydantic import BaseModel, validate_arguments
+from pydantic import BaseModel, validate_call
 import asyncio
 
 import invertedai as iai
@@ -18,6 +18,8 @@ from invertedai.common import (
     InfractionIndicators,
     AgentAttributes,
     TrafficLightStatesDict,
+    LightRecurrentStates,
+    LightRecurrentState,
 )
 
 
@@ -41,20 +43,25 @@ class DriveResponse(BaseModel):
     is_inside_supported_area: List[
         bool
     ]  #: For each agent, indicates whether the predicted state is inside supported area.
+    traffic_lights_states: Optional[TrafficLightStatesDict] #: Traffic light states for the full map, each key-value pair corresponds to one particular traffic light.
+    light_recurrent_states: Optional[LightRecurrentStates] #: Light recurrent states for the full map, each element corresponds to one light group.
+    api_model_version: str  # Model version used for this API call
 
 
-@validate_arguments
+@validate_call
 def drive(
     location: str,
     agent_states: List[AgentState],
     agent_attributes: List[AgentAttributes],
-    recurrent_states: List[RecurrentState],
+    recurrent_states: Optional[List[RecurrentState]] = None,
     traffic_lights_states: Optional[TrafficLightStatesDict] = None,
+    light_recurrent_states: Optional[LightRecurrentStates] = None,
     get_birdview: bool = False,
     rendering_center: Optional[Tuple[float, float]] = None,
     rendering_fov: Optional[float] = None,
     get_infractions: bool = False,
     random_seed: Optional[int] = None,
+    api_model_version: Optional[str] = None
 ) -> DriveResponse:
     """
     Parameters
@@ -64,14 +71,16 @@ def drive(
 
     agent_states:
         Current states of all agents.
-        The state must include x: [float], y: [float] corrdinate in meters
+        The state must include x: [float], y: [float] coordinate in meters
         orientation: [float] in radians with 0 pointing along x and pi/2 pointing along y and
         speed: [float] in m/s.
 
     agent_attributes:
         Static attributes of all agents.
         List of agent attributes. Each agent requires, length: [float]
-        width: [float] and rear_axis_offset: [float] all in meters.
+        width: [float] and rear_axis_offset: [float] all in meters. agent_type: [str],
+        currently supports 'car' and 'pedestrian'.
+        waypoint: optional [Point], the target waypoint of the agent.
 
     recurrent_states:
         Recurrent states for all agents, obtained from the previous call to
@@ -94,21 +103,27 @@ def drive(
     traffic_lights_states:
        If the location contains traffic lights within the supported area,
        their current state should be provided here. Any traffic light for which no
-       state is provided will be ignored by the agents.
+       state is provided will have a state generated by iai.
+
+    light_recurrent_states:
+       Light recurrent states for all agents, obtained from the previous call to
+        :func:`drive` or :func:`initialize`.
+       Specifies the state and time remaining for each light group in the map.
+       If manual control of individual traffic lights is desired, modify the relevant state(s) 
+       in traffic_lights_states, then pass in light_recurrent_states as usual.
 
     random_seed:
         Controls the stochastic aspects of agent behavior for reproducibility.
 
+    api_model_version:
+        Optionally specify the version of the model. If None is passed which is by default, the best model will be used.
     See Also
     --------
     :func:`initialize`
     :func:`location_info`
     :func:`light`
+    :func:`blame`
     """
-    if len(agent_states) != len(agent_attributes):
-        raise InvalidInput("Incompatible Number of Agents in either 'agent_states' or 'agent_attributes'.")
-    if len(agent_states) != len(recurrent_states):
-        raise InvalidInput("Incompatible Number of Agents in either 'agent_states' or 'recurrent_states'.")
 
     if should_use_mock_api():
         agent_states = [mock_update_agent_state(s) for s in agent_states]
@@ -130,20 +145,21 @@ def drive(
         else:
             return input_data
 
-    recurrent_states = (
-        _tolist(recurrent_states) if recurrent_states is not None else None
-    )  # AxTx2x64
+    recurrent_states = _tolist(recurrent_states) if recurrent_states is not None else None
     model_inputs = dict(
         location=location,
         agent_states=[state.tolist() for state in agent_states],
         agent_attributes=[state.tolist() for state in agent_attributes],
-        recurrent_states=[r.packed for r in recurrent_states],
+        recurrent_states=[r.packed for r in recurrent_states] if recurrent_states is not None else None,
         traffic_lights_states=traffic_lights_states,
+        light_recurrent_states=[light_recurrent_state.tolist() for light_recurrent_state in light_recurrent_states] 
+        if light_recurrent_states is not None else None,
         get_birdview=get_birdview,
         get_infractions=get_infractions,
         random_seed=random_seed,
         rendering_center=rendering_center,
-        rendering_fov=rendering_fov
+        rendering_fov=rendering_fov,
+        model_version=api_model_version
     )
     start = time.time()
     timeout = TIMEOUT
@@ -169,53 +185,67 @@ def drive(
                 if response["infraction_indicators"]
                 else [],
                 is_inside_supported_area=response["is_inside_supported_area"],
+                api_model_version=response["model_version"],
+                traffic_lights_states=response["traffic_lights_states"]
+                if response["traffic_lights_states"] is not None 
+                else None,
+                light_recurrent_states=[
+                    LightRecurrentState(state=state_arr[0], time_remaining=state_arr[1]) 
+                    for state_arr in response["light_recurrent_states"]
+                ] 
+                if response["light_recurrent_states"] is not None 
+                else None
             )
 
             return response
         except APIConnectionError as e:
             iai.logger.warning("Retrying")
             if (
-                timeout is not None and time.time() > start + timeout
+                    timeout is not None and time.time() > start + timeout
             ) or not e.should_retry:
                 raise e
 
 
-@validate_arguments
+@validate_call
 async def async_drive(
     location: str,
     agent_states: List[AgentState],
     agent_attributes: List[AgentAttributes],
-    recurrent_states: List[RecurrentState],
+    recurrent_states: Optional[List[RecurrentState]] = None,
     traffic_lights_states: Optional[TrafficLightStatesDict] = None,
+    light_recurrent_states: Optional[LightRecurrentStates] = None,
     get_birdview: bool = False,
     rendering_center: Optional[Tuple[float, float]] = None,
     rendering_fov: Optional[float] = None,
     get_infractions: bool = False,
     random_seed: Optional[int] = None,
+    api_model_version: Optional[str] = None
 ) -> DriveResponse:
     """
     A light async version of :func:`drive`
     """
+
     def _tolist(input_data: List):
         if not isinstance(input_data, list):
             return input_data.tolist()
         else:
             return input_data
 
-    recurrent_states = (
-        _tolist(recurrent_states) if recurrent_states is not None else None
-    )  # AxTx2x64
+    recurrent_states = _tolist(recurrent_states) if recurrent_states is not None else None
     model_inputs = dict(
         location=location,
         agent_states=[state.tolist() for state in agent_states],
         agent_attributes=[state.tolist() for state in agent_attributes],
-        recurrent_states=[r.packed for r in recurrent_states],
+        recurrent_states=[r.packed for r in recurrent_states] if recurrent_states is not None else None,
         traffic_lights_states=traffic_lights_states,
+        light_recurrent_states=[light_recurrent_state.tolist() for light_recurrent_state in light_recurrent_states] 
+        if light_recurrent_states is not None else None,
         get_birdview=get_birdview,
         get_infractions=get_infractions,
         random_seed=random_seed,
         rendering_center=rendering_center,
-        rendering_fov=rendering_fov
+        rendering_fov=rendering_fov,
+        model_version=api_model_version
     )
     response = await iai.session.async_request(model="drive", data=model_inputs)
 
@@ -236,6 +266,16 @@ async def async_drive(
         if response["infraction_indicators"]
         else [],
         is_inside_supported_area=response["is_inside_supported_area"],
+        api_model_version=response["model_version"],
+        traffic_lights_states=response["traffic_lights_states"] 
+        if response["traffic_lights_states"] is not None 
+        else None,
+        light_recurrent_states=[
+            LightRecurrentState(state=state_arr[0], time_remaining=state_arr[1]) 
+            for state_arr in response["light_recurrent_states"]
+        ] 
+        if response["light_recurrent_states"] is not None 
+        else None
     )
 
     return response
